
//'WWWW'WWW''''''''WW''''''''WWW''WW''W'''''''''''''''''''''''''WWW'''''''
//WW  W  WW        WW         WW  WW  W                          WW       
//WW     WW   WW   WWW   WWW  WW  WW  W  WWW W WW  WWWW     WWW  WW   WWW 
//WW WW  WW  WW W  WW W W WW  WW  WW  W W WW WW   WWW      WW W  WW  W WW 
//WW  W  WW  WW W  WW W W WW  WW  WW W  W WW WW     WWW    WW    WW  W WW 
// WWWW WWWW  WW   WWW   WWW WWWW  WW    WWW WW   WWWW      WWW WWWW  WWW 
//                                                                        
//''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''

#include "GlobalVars.h"
#include "Pane.h"
#include "PreviewQuad.h"
#include "CameraList.h"

GlobalVars::GlobalVars(){
  sceneFile = "scene.iv";
  imgFile = "image.png";
  inImgFile.clear();
  imgDim[X] = 400;
  imgDim[Y] = 300;
  imgMin[X] = -imgDim[X]/(double)imgDim[Y]; //-4.0/3.0;
  imgMax[X] = imgDim[X]/(double)imgDim[Y];  // 4.0/3.0;
  imgMin[Y] = -1.0;
  imgMax[Y] = 1.0;
  headLightPos = 0;
  renderQuit = 0;
  octreeDebug = 0;
  octreeMaxDepth = 8;
  octreeMaxItems = 10;
  algorithm = ALG_PM;
  numSamples = 1;
  timeSamples = 0;
  maxDepth = 12;
  luminanceMean = -1.0;
  times.render = 0;
  times.renderTicks = 0;
  times.totalTime = 0;
  times.totalTicks = 0;
  times.octreeCreation = 0;
  times.photonShooting = 0;
  times.photonBalancing = 0;
  times.photonIrradiance = 0;
  lightSamples = -1;
  totalLightSamples = 0;
  debugPixel[X] = debugPixel[Y] = -1;
  debugRange = -1;
  animateScene = 0;
  areaLights = 1;
  automaticUpdate = 0;
  createBackup = 1;
  allowReflection = 1;
  pane = NULL;
  cam = NULL;
  root = NULL;
  scene = NULL;
  trans = NULL;
  camList = NULL;
  xtViewer = NULL;
  preview = NULL;
  sceneSwitch = NULL;
  crossSwitch = NULL;
  cmdLine = "";
  unitsPerMeter = -1;
  unitsPerMeter3 = -1;
  metersPerUnit = -1.0;
  metersPerUnit2 = -1.0;
  windowBG.setValue(0.5, 0.0, 0.5);
  worldBG.setValue(0.0, 0.0, 0.0);
  gPhotonEstimate = -1;
  cPhotonEstimate = -1;
  iPhotonEstimate = 10;
  vPhotonEstimate = -1;
  numPhotonsFireMax = (float)LONG_MAX;
  numPhotons = 90000;
  numCPhotons = 2000;
  numIPhotons = 0;
  numVPhotons = 0;
  approximate = 0;
  irradCacheError = 0.15;
  showPhotons = 0;
  toneMapping = TM_WARDCBSF;
  renderFlags = 0;
  gatherRays = 200;
  precomputeIrr = 1;
  fastTransparency = 0;
  numThreads = -1;
  nonRandomPixels = 0;
  quasiRandom = 1;
  ambientResolution = 64.0;
  refractionIndex = 1.7;
  tempInt = 0;
  tempFloat = 0.0;
  tempFloat2 = 0.0;
  tempFloat3 = 0.0;
  maxRelError=0.0;
  floatMap = 0;
  timeVal = -1.0;
  timeMin = timeMax = timeVal;
  frames = 100;
  framesSegment[0] = framesSegment[1] = 0;
  rrStartDepth = 8;
  sceneHasShininess = 0;
  sceneHasLights = 0;
  seeSingleSided = 1;
  useGlassModel = 1;
  renderIv = 0;
  ivOutFile = "pane.iv";
  useDimensions = 3;
  figureMode = 0;
  gamma = 2.2;
  exposure = 1.0;
  showNormals = 0;
  comment = "";
  environMapFile.clear();
  envMapRotation = 0.0;
  aperture = 0.0;
  transpScale = 0.75;
  sampleMap.clear();
  useEmisFluxDensity = 0;
  computeNormals = 0;
  useGlare = 0;
  srandSeed = 1;
  simpleOctree = 0;
  silent = 0;
  prepass = 1;
  makeMatte = 0;
  attenuation.setValue(0.0, 0.0, 0.0);
  useIndirectImpSampling = 0;
  bumpMapping = 0;
  bumpScale = 1.0;
  useIrradCache = 1;
  heightAngle = 45.0/360.0*2.0*M_PI; // radians
  sigma_t = 0.5;
  albedo = 0.5;
  stepSize = 0.0;
  scalarDataFile.clear();
  scalarDataFile2.clear();
  scalarDataDim[0] = scalarDataDim[1] = scalarDataDim[2] = 100;
  scalarDataHeader = 512;
  scalarDataMult = 1.0;
  scalarDataMult2 = 1.0;
  scalarDataMax = SbVec3f(100, 100, 100);
  scalarColor = SbColor(1,1,1);
  envLockSamples = 0;
  localIllum = 0;
  ambientIllum = 0;
  accelType = ACC_OCTREE;
  depthImage = DEPTH_NONE;
  compressEst = 1.0;
}

GlobalVars::~GlobalVars(){
  delete preview;
  delete camList;
}

// Read command line options and set variables
void GlobalVars::readOpts(int argc, char *argv[]){
  int c;
  int option_index = 0;

  for (int i=0; i<argc; i++)
    cmdLine = cmdLine + argv[i] + " ";

  static struct option long_options[] = {
    {"accel",         1, 0, 'R'},
    {"albedo",        1, 0, 61 },
    {"algorithm",     1, 0, 'a'},
    {"ambientres",    1, 0, 25 },
    {"ambientillum",  0, 0, 257},
    {"animate",       0, 0,  6 },
    {"aperture",      1, 0, 42 },
    {"approx",        0, 0, 13 },
    {"attenuation",   1, 0, 53 },
    {"autoupdate",    0, 0,  8 },
    {"bumpmap",       1, 0, 57 },
    {"bumpscale",     1, 0, 58 },
    {"computenorm",   0, 0, 'c'},
    {"dims",          1, 0, 36 },
    {"depthimage",    2, 0, 259},
    {"estimate",      1, 0, 12 },
    {"exposure",      1, 0, 40 },
    {"cestimate",     1, 0, 15 },
    {"vestimate",     1, 0, 60 },
    {"comment",       1, 0, 39 },
    {"emisdensity",   0, 0, 46 },
    {"environment",   1, 0, 'e'},
    {"envlock",       0, 0, 'L'},
    {"envrotate",     1, 0, 52 },
    {"fasttrans",     0, 0, 22 },
    {"figures",       0, 0, 37 },
    {"flags",         1, 0, 19 },
    {"floatmap",      0, 0, 29 },
    {"frames",        1, 0, 32 },
    {"framessegment", 1, 0, 62 },
    {"gatherrays",    1, 0, 20 },
    {"gamma",         1, 0, 38 },
    {"glare",         1, 0, 'G'},
    {"headlight",     2, 0, 'l'},
    {"help",          0, 0, 'h'},
    {"icerror",       1, 0, 16 },
    {"image",         1, 0, 'i'},
    {"imagedim",      1, 0, 'd'},
    {"iestimate",     1, 0, 50 },
    {"indimpsamp",    0, 0, 54 },
    {"localillum",    0, 0, 258},
    {"maxdepth",      1, 0, 'm'},
    {"maxrelerror",   1, 0, 30 },
    {"modeltrans",    1, 0, 43 },
    {"noarea",        0, 0, 'n'},
    {"nobackup",      0, 0,  9 }, 
    {"noshiny",       0, 0, 10 }, 
    {"notonemapping", 0, 0, 18 },
    {"noprecompute",  0, 0, 21 },
    {"noprepass",     0, 0, 49 },
    {"noquasi",       0, 0, 'q'},
    {"norandompixels",0, 0, 23 },
    {"nosinglesided", 0, 0, 34 },
    {"noglass",       0, 0, 35 },
    {"output",        1, 0, 'o'},
    {"photons",       1, 0, 'p'},
    {"cphotons",      1, 0, 14 },
    {"importons",     1, 0, 51 },
    {"vphotons",      1, 0, 59 },
    {"fphotons",      1, 0, 'M'},
    {"pixel",         1, 0, 'g'},
    {"samples",       1, 0, 's'},
    {"lightsamples",  1, 0,  2 },
    {"lummean",       1, 0, 'y'},
    {"matte",         0, 0, 41 },
    {"silent",        0, 0, 45 },
    {"showoctree",    0, 0,  1 },
    {"showphotons",   0, 0, 17 },
    {"octreedepth",   1, 0,  4 },
    {"octreeitems",   1, 0,  5 },
    {"range",         1, 0, 24 },
    {"render",        0, 0, 'r'},
    {"renderiv",      0, 0, 'x'},
    {"rindex",        1, 0, 26 },
    {"rrdepth",       1, 0, 33 },
    {"samplemap",     1, 0, 44 },
    {"sigma_t",       1, 0, 55 },
    {"stepsize",      1, 0, 56 },
    {"simpleoctree",  0, 0, 48 },
    {"srand",         1, 0, 47 },
    {"time",          1, 0, 31 },
    {"timesamples",   1, 0, 'S'},
    {"tonemapping",   1, 0, 'T'},
    {"numthreads",    1, 0, 'j'},
    {"unitspermeter", 1, 0, 'u'}, 
    {"worldbg",       1, 0,  3 },
    {"windowbg",      1, 0,  7 },
    {"version",       0, 0, 'v'},
    {"3ddata",        1, 0, 'A'},
    {"3ddata2",       1, 0, 'B'},
    {"3ddim",         1, 0, 'C'},
    {"3dheader",      1, 0, 'D'},
    {"3dmult",        1, 0, 'E'},
    {"3dmult2",       1, 0, 'F'},
    {"3dcolor",       1, 0, 'J'},
    {"3dmax",         1, 0, 'K'},
    {"compressEstimate",1,0,'N'},
    
    {"tempint",       1, 0, 27 },
    {"tempfloat",     1, 0, 28 },
    {"tempfloat2",    1, 0, 'H'},
    {"tempfloat3",    1, 0, 'I'},
    {0, 0, 0, 0}
  };
    
  while (1){
    c = getopt_long(argc, argv, "a:cd:e:g:G:hi:j:l::m:no:p:qrs:S:T:u:vxy:"
		    "A:B:C:D:E:LM:R:", long_options,
		    &option_index);

    if (c == -1)
      break;

    double r, g, b;
    char *ptr;
    switch (c){
    case 1:
      octreeDebug = 1;
      break;

    case 2:
      lightSamples = atoi(optarg);
      break;

    case 3:
      ptr = optarg;
      r = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      g = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      b = atof(ptr);
      worldBG.setValue(r, g, b);
      break;

    case 4:
      octreeMaxDepth = atoi(optarg);
      break;

    case 5:
      octreeMaxItems = atoi(optarg);
      break;

    case 6:
      animateScene = 1;
      automaticUpdate = 1;
      break;

    case 7:
      ptr = optarg;
      r = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      g = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      b = atof(ptr);
      windowBG.setValue(r, g, b);
      break;
	
    case 8:
      automaticUpdate = 1;
      break;

    case 9:
      createBackup = 0;
      break;

    case 10:
      allowReflection = 0;
      numCPhotons = 0;
      break;

    case 12:
      gPhotonEstimate = atoi(optarg);
      if (gPhotonEstimate<8)
	cerr << "WARNING: a photon estimate less than 8 results "
	     << "in zero irradiance!\n"
	     << "Fix: Use a higher estimate, or more photons.\n";
      break;

    case 13:
      approximate = 1;
      break;

    case 14:
      numCPhotons = atoi(optarg);
      break;

    case 15:
      cPhotonEstimate = atoi(optarg);
      if (cPhotonEstimate<8)
	cerr << "WARNING: a photon estimate less than 8 results "
	     << "in zero irradiance!\n"
	     << "Fix: Use a higher estimate, or more photons.\n";
      break;

    case 16:
      irradCacheError = atof(optarg);
      break;

    case 17:
      showPhotons = 1;
      break;
      
    case 18:
      toneMapping = TM_NONE;
      break;

    case 19:
      renderFlags = atoi(optarg);
      break;

    case 20:
      gatherRays = atoi(optarg);

      // ensure it doesnt exceed lookup table max size
      if (gatherRays>30000){
	gatherRays = 30000;
	cerr << "Warning: Too many gatherRays. Using 30000.\n";
      }
      break;

    case 21:
      precomputeIrr = 0;
      break;

    case 22:
      fastTransparency = 1;
      break;

    case 23:
      nonRandomPixels = 1;
      break;

    case 24:
      debugRange = atoi(optarg);
      break;

    case 25:
      ambientResolution = atof(optarg);
      break;

    case 26:
      refractionIndex = atof(optarg);
      break;

    case 27:
      tempInt = atoi(optarg);
      break;

    case 28:
      tempFloat = atof(optarg);
      break;

    case 29:
      floatMap = 1;
      break;

    case 30:
      maxRelError = atof(optarg);
      break;

    case 31:
      timeVal = atof(optarg);
      break;

    case 32:
      frames = atoi(optarg);
      break;

    case 33:
      rrStartDepth = atoi(optarg);
      break;

    case 34:
      seeSingleSided = 0;
      break;

    case 35:
      useGlassModel = 0;
      break;

    case 36:
      useDimensions = atoi(optarg);
      break;

    case 37:
      figureMode = 1;
      break;

    case 38:
      gamma = atof(optarg);
      break;

    case 39:
      comment = optarg;
      break;

    case 40:
      exposure = atof(optarg);
      break;

    case 41:
      makeMatte = 1;
      break;

    case 42:
      aperture = atof(optarg);
      break;

    case 43:
      transpScale = atof(optarg);
      break;
      
    case 44:
      sampleMap = optarg;
      break;
     
    case 45:
      silent = 1;
      break;

    case 46:
      useEmisFluxDensity = 1;
      break;

    case 47:
      srandSeed = atoi(optarg);
      break;

    case 48:
      simpleOctree = 1;
      break;

    case 49:
      prepass = 0;
      break;

    case 50:
      iPhotonEstimate = atoi(optarg);
      break;

    case 51:
      numIPhotons = atoi(optarg);
      break;

    case 52:
      envMapRotation = atof(optarg);
      break;

    case 53:
      ptr = optarg;
      r = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      g = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      b = atof(ptr);
      attenuation.setValue(r, g, b);

      // fold log into attenuation
      attenuation[0] = log(attenuation[0]);
      attenuation[1] = log(attenuation[1]);
      attenuation[2] = log(attenuation[2]);
      break;

    case 54:
      useIndirectImpSampling = 1;
      break;

    case 55:
      sigma_t = atof(optarg);
      break;

    case 56:
      stepSize = atof(optarg);
      break;

    case 57:
      bumpMapping = atoi(optarg);
      break;

    case 58:
      bumpScale = atof(optarg);
      break;

    case 59:
      numVPhotons = atoi(optarg);
      break;

    case 60:
      vPhotonEstimate = atoi(optarg);
      if (vPhotonEstimate<8)
	cerr << "WARNING: a photon estimate less than 8 results "
	     << "in zero irradiance!\n"
	     << "Fix: Use a higher estimate, or more photons.\n";
      break;

    case 61:
      albedo = atof(optarg);
      break;

    case 62:
      framesSegment[0] = atoi(optarg);
      framesSegment[1] = atoi(strchr(optarg, '/')+1);
      if (framesSegment[0] <= 0 || framesSegment[0] > framesSegment[1]){
	cerr << "Invalid segment selection: " << optarg << endl;
	exit(1);
      }
      break;

    // careful how high you go: 'A'=65

    case 'A':
      scalarDataFile = optarg;
      break;

    case 'B':
      scalarDataFile2 = optarg;
      break;

    case 'C':
      scalarDataDim[0] = atoi(optarg);
      ptr = strchr(optarg, 'x');
      if (ptr == NULL)
	scalarDataDim[1] = scalarDataDim[2] = scalarDataDim[0];
      else {
	ptr++;
	scalarDataDim[1] = atoi(ptr);
	ptr = strchr(ptr, 'x')+1;
	scalarDataDim[2] = atoi(ptr);
      }
      if (scalarDataDim[0] < 1 || scalarDataDim[1] < 1 || scalarDataDim[2] < 1){
	cerr << "Invalid data set dimensions: " << optarg << endl;
	exit(1);
      }
      break;

    case 'D':
      scalarDataHeader = atoi(optarg);
      break;

    case 'E':
      scalarDataMult = atof(optarg);
      break;

    case 'F':
      scalarDataMult2 = atof(optarg);
      break;
      
    case 'G':
      useGlare = atoi(optarg);
      break;

    case 'H':
      tempFloat2 = atof(optarg);
      break;
      
    case 'I':
      tempFloat3 = atof(optarg);
      break;

    case 'J':
      ptr = optarg;
      scalarColor[0] = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      scalarColor[1] = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      scalarColor[2] = atof(ptr);
      break;

    case 'K':
      ptr = optarg;
      scalarDataMax[0] = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      scalarDataMax[1] = atof(ptr);
      ptr = strchr(ptr, ',')+1;
      scalarDataMax[2] = atof(ptr);
      break;
      
    case 'L':
      envLockSamples = 1;
      break;

    case 'M':
      numPhotonsFireMax = atof(optarg);
      break;

    case 'N':
      compressEst = atof(optarg);
      break;

    case 'R':
      accelType = (GlobalVars::accel_t)atoi(optarg);
      if (accelType == ACC_KDTREE){
	octreeMaxItems = 1;
	octreeMaxDepth = 30;
      }
      
    case 'S':
      timeSamples = atoi(optarg);
      break;

    case 'T':
      if (strcmp(optarg, "none") == 0 || atoi(optarg)==TM_NONE && isdigit(optarg[0]))
	toneMapping = TM_NONE;
      else if (strcmp(optarg, "wardcbsf") == 0 || atoi(optarg)==TM_WARDCBSF)
	toneMapping = TM_WARDCBSF; // Contrast-based scale-factor
      else if (strcmp(optarg, "wardhist") == 0 || atoi(optarg)==TM_WARDHIST)
	toneMapping = TM_WARDHIST; // Histogram adjustment
      else if (strcmp(optarg, "wardhuman") == 0 || atoi(optarg)==TM_WARDHUMAN)
	toneMapping = TM_WARDHUMAN; // Histogram adjustment with human limitations
      else if (strcmp(optarg, "nonlinear") == 0 || atoi(optarg)==TM_NONLINEAR)
	toneMapping = TM_NONLINEAR; // Non-linear
      else {
	cerr << "Error: Unknown tonemapping: " << optarg << "\n";
	exit(4);
      }
      break;

    case 257:
      ambientIllum = 1;
      sceneHasLights++;
      algorithm=0;
      numCPhotons=0;
      areaLights=0;
      break;

    case 258:
      localIllum = 1;
      algorithm=0;
      numCPhotons=0;
      areaLights=0;
      break;

    case 259:
      depthImage=optarg?(depthImage_t)atoi(optarg):DEPTH_AVERAGE;
      break;
      
    case 'a':
      algorithm = atoi(optarg);
      if (algorithm == 4){
	toneMapping = TM_NONE;
	gamma = 1.0;
	sceneHasLights++;
	numCPhotons=0;
	areaLights=0;
      }
      break;

    case 'c':
      computeNormals = 1;
      break;

    case 'd':
      imgDim[X] = atoi(optarg);
      imgDim[Y] = atoi(strchr(optarg, 'x')+1);
      if (imgDim[X] <= 0 || imgDim[Y] <= 0){
	cerr << "Invalid image dimensions: " << optarg << endl;
	exit(1);
      }

      // reset imgMax/imgMin
      if (imgDim[X]>imgDim[Y]){
	imgMin[X] = -imgDim[X]/(double)imgDim[Y]; //-4.0/3.0;
	imgMax[X] = imgDim[X]/(double)imgDim[Y];  // 4.0/3.0;
	imgMin[Y] = -1.0;
	imgMax[Y] = 1.0;
      }
      else {
	imgMin[X] = -1.0;
	imgMax[X] = 1.0;
	imgMin[Y] = -imgDim[Y]/(double)imgDim[X]; //-4.0/3.0;
	imgMax[Y] = imgDim[Y]/(double)imgDim[X];  // 4.0/3.0;
      }
      break;

    case 'e':
      environMapFile = optarg;
      break;
	
    case 'g':
      debugPixel[X] = atoi(optarg);
      debugPixel[Y] = atoi(strchr(optarg, ',')+1);
      break;

    case 'h':
      printUsage();
      exit(1);
      break;

    case 'i':
      inImgFile = optarg;
      break;

    case 'j':
      numThreads = atoi(optarg);
      break;

    case 'l':
      headLightPos = optarg?atoi(optarg):1;
      if (headLightPos)
	sceneHasLights++;
      break;

    case 'm':
      maxDepth = atoi(optarg);
      break;
	
    case 'n':
      areaLights = 0;
      break;

    case 'o':
      imgFile = optarg;
      ivOutFile = optarg;
      break;

    case 'p':
      numPhotons = atoi(optarg);
      break;

    case 'q':
      quasiRandom = 0;
      break;
      
    case 'r':
      renderQuit = 1;
      break;
	
    case 's':
      numSamples = atoi(optarg);
      break;

    case 't':
      renderQuit = 2; // trace pixel
      break;
	
    case 'v':
      printUsage(1);
      exit(1);
      break;

    case 'u':
      setUnitsPerMeter(atof(optarg));
      break;

    case 'x':
      renderIv = 1;
      break;

    case 'y':
      luminanceMean = atof(optarg);
      break;

    default:
      cerr << flush;
      fprintf(stderr, "Error: Unknown command line option.\n"
	      "(getopt returned character code 0%o ?""?)\n", c);
      exit(2);
    }
  }

  // list unknown options
  if (optind < argc) {
    sceneFile = argv[optind++];

    if (optind < argc){
      cerr << "non-option elements: " << flush;
      while (optind < argc)
	cerr << argv[optind++];
      cerr << endl;
      exit(1);
    }
  }

  return;
}

void GlobalVars::printUsage(int version){
    
  // revision #
  const char *revision = pane->getVersion();
  char info2[] = "$Date: 2006/07/16 06:50:49 $";
  char *p = strchr(info2, ' ')+1;
  char *date = strsep(&p, " ");

  if (version){
    cerr <<
      "Pane v" << revision << " by Kevin Beason " << date << "\n";
    return;
  }
    
  cerr <<
    "\n"
    "Pane v" << revision << " " << date << "\n"
    "by Kevin Beason (beason@cs.fsu.edu)\n"
    "Source and more information available at http://www.csit.fsu.edu/~beason/pane/\n"
    "\n"
    "Ray traces an Inventor scene (Default: scene.iv) storing output in\n"
    "an image (Default: image.png).\n"
    "\n"
    "Usage: pane [OPTION]... [IV FILE]\n"
    "\n"
    "Example:\n"
    "\tpane /usr/share/data/models/star.iv\n"
    "\n"
    "Other examples:\n"
    "\n"
    "\t# non-interactive render mode\n"
    "\tpane -r star.iv\n"
    "\n"
    "\t# render geometry instead of image\n"
    "\tivcat -f scene.iv |ivfix > renderme.iv\n"
    "\tpane -c -x renderme.iv\n"
    "\n"
    "\t# photon map approximation\n"
    "\tpane --approx -r star.iv\n"
    "\n"
    "\t# direct lighting only\n"
    "\tpane -a0 -r star.iv\n"
    "\n"
    "\t# high quality render\n"
    "\tpane --photons=1000000 -d1024x768 -s16 -r star.iv\n"
    "\n"
    "\t# render animation, using camera positions in camlist.iv\n"
    "\tpane --animate --frames=300 -d512x384 -s4 -r star.iv\n"
    "\n"
    "\t# render single frames, using first, middle, and\n"
    "\t# last camera positions in camlist.iv\n"
    "\tpane --time=0   -r star.iv\n"
    "\tpane --time=0.5 -r star.iv\n"
    "\tpane --time=1.0 -r star.iv\n"
    "\n"
    "\t# render using HDR image for image based lighting\n"
    "\tpane --environment=grace_probe.hdr -r star.iv\n"
    "\n"
    "\t# render using Monte Carlo Path Tracing\n"
    "\tpane -a2 -s1000 -r star.iv\n"
    "\n"
    "\t# re-expose an image (to brighten or darken)\n"
    "\tpane -i image.hdr --exposure=1.2 --lummean=100 -o corrected.png -r\n"
    "\n"
    "\t# view pane command used to render\n"
    "\tidentify -format '%c' image.png      # for image (requires imagemagick)\n"
    "\tivcat pane.iv | head -30             # for scene\n"
    "\n"
    "Options:\n"
    "\t-R, --accel=NUM         0=Octree, 1=KdTree\n"
    "\t-a, --algorithm=NUM     0=RT 1=PM 2=MCPT w/EL 3=MCPT 4=FPS (1)\n"
    "\t    --ambientillum      Use diffuse color as shaded color.\n"
    "\t    --ambientres=NUM    maximum ambient resolution factor (64)\n"
    "\t    --animate           animate camera through camlist.iv\n"
    "\t    --aperture=NUM      angle of DOF blur in degrees (0)\n"
    "\t    --approx            render using global photon map\n"
    "\t    --attenuation=R,G,B fraction of light at 1 meter in glass (1.0, 1.0, 1.0)\n"
    "\t    --autoupdate        automatically update octree on scene change\n"
    "\t    --bumpmap=NUM       generate bump maps from textures (0)\n"
    "\t                        0=texture 1=texture+bump 2=bump only\n"
    "\t    --bumpscale=NUM     scale of bump map height (1.0) \n"
    "\t    --estimate          number of photons in irradiance estimate\n"
    "\t    --cestimate         number of photons in caustic irradiance estimate\n"
    "\t    --compressEstimate=NUM scale to shrink photon volume into disc\n"
    "\t    --vestimate         number of photons in volume irradiance estimate\n"
    "\t    --comment=STRING    comment to insert in output file\n"
    "\t-c, --computenorm       compute normals for indexed face sets\n"
    "\t    --dims=NUM          dimensions to allow light to flow (3)\n"
    "\t    --depthimage=TYPE   output depth image. 0=none 1=nearest 2=average\n"
    "\t    --emisdensity       use emissiveColor as density, not flux\n"
    "\t-e, --environment=FILE  use environment map (.float, .hdr) for RNL\n"
    "\t                        do --environment=-1 to use worldBG as HDR map\n"
    "\t    --envlock           lock the samples for environment IBL\n"
    "\t    --envrotate=NUM     rotate environment map about Y axis by NUM degrees\n"
    "\t    --exposure=NUM      exposure to use (brightness) (1.0)\n"
    "\t    --fasttrans         use fast transparency (stipple alpha) in viewer\n"
    "\t    --figures           for figure purposes - shows photon paths and more\n"
    "\t    --flags=NUM         rendering components sum: approx=1,spec=2,\n"
    "\t                        direct=4,emitted=8,caustic=16,mdiffuse=32\n"
//    "\t    --floatmap          writes Pfm file instead of binary PPM\n"
    "\t    --frames=NUM        number of frames of animation to render (100)\n"
    "\t    --framessegment=X/Y divide frames into Y segments, and render segment X (0/0)\n"
    "\t    --gamma=NUM         gamma to use for gamma correction (2.2)\n"
    "\t    --gatherrays=NUM    number of rays to use in final gather (200)\n"
    "\t    --glare=TYPE        render glare using filter\n"
    "\t                        1=shirley glare,[photopic|mesopic|scotopic].hdr\n"
    "\t                        2=nakamae bloom\n"
    "\t-g, --pixel=X,Y         when tracing ray, trace through pixel (X,Y)\n"
    "\t-h, --help              print this help information\n"
    "\t-i, --image=FILE        read .hdr image to framebuffer from FILE\n"
    "\t    --iestimate=NUM     number of photons to use in importance estimate (10)\n"
    "\t-j, --threads=NUM       number of threads to use (1)\n"
    "\t-l, --headlight=NUM     turns on NUM (optional) headlight(s)\n"
    "\t    --icerror=NUM       error used in irradiance cache (0.15)\n"
    "\t-d, --imagedim=XxY      dimensions of output image\n"
    "\t    --indimpsamp        use indirect importance sampling using photon map\n"
    "\t    --lightsamples=NUM  number of times to sample area lights (100)\n"
    "\t    --localillum        Use local illum, i.e. -a0 & visibility=1\n"
    "\t-y, --lummean=NUM       mean luminance value for tonemapping (-1.0)\n"
    "\t-m, --maxdepth=NUM      maximum recursion depth for rays (12)\n"
    "\t    --maxrelerror=NUM   attempts to limit error in pixel samples (inf)\n"
    "\t    --matte             draw background as worldBG for matting\n"
    "\t    --modeltrans=NUM    scalefactor for model's transparency (0.75)\n"
    "\t-n, --noarea            disable area lights\n"
    "\t    --nobackup          don't move file if already exists\n"
    "\t    --noglass           Use tissue paper model for transparency\n"
    "\t    --noshiny           disable shiny surfaces\n"
    "\t    --noprecompute      do not precompute irradiance at each photon\n"
    "\t    --noprepass         do not do indirect illumination prepass\n"
    "\t-q, --noquasi           do not use quasi random numbers\n"
    "\t    --norandompixels    call srand48(i*j) before each pixel calc.\n"
    "\t    --nosinglesided     does not draw surfaces facing away from camera\n"
//    "\t    --notonemapping     turns off tone mapping\n"
    "\t-T, --tonemapping=TYPE  none|wardcbsf|wardhist|wardhuman|nonlinear (wardcbsf)\n"
    "\t    --showoctree        display cells of octree\n"
    "\t    --showphotons       display stored photons in photon map\n"
    "\t    --octreedepth=NUM   depth of octree (8)\n"
    "\t    --octreeitems=NUM   max items per cell (10)\n"
    "\t-o, --output=FILE       writes output to FILE (image.png)\n"
    "\t-p, --photons=NUM       number of photons to use (for -a1) (90000)\n"
    "\t    --cphotons=NUM      number of caustic photons to use (for -a1) (1000)\n"
    "\t    --importons=NUM     number of importance photons to use (0)\n"
    "\t    --vphotons=NUM      number of volume photons to use (0)\n"
    "\t-M, --fphotons=NUM      maximum number of photons to fire (LONG_MAX)\n"
    "\t    --range=NUM         renders target pixel +/- NUM pixels when rendering\n"
    "\t-r, --render            renders output without creating window\n"
    "\t-x, --renderiv          renders the scene and saves in pane.iv\n"
    "\t    --rindex=NUM        refraction index for transparent materials (1.7)\n"
    "\t    --rrdepth=NUM       depth at which to start russian roulette (8)\n"
    "\t-s, --samples=NUM       number of rays per pixel\n"
    "\t    --samplemap=FILE    PGM containing map of region to sample.\n"
    "\t    --silent            turns off beep when render is done.\n"
    "\t    --simpleoctree      use midpoint subdivision instead of optimizing\n"
    "\t    --srand=NUM         an integer to seed random number generators (0)\n"
    "\t    --time=NUM          sets world time for animations in [0,1] (-1.0)\n"
    "\t-S, --timesamples=NUM   number of time samples to use for motion blur (0)\n"
    "\t-u, --unitspermeter     ratio of world units to real world meter\n"
    "\t-v, --version           prints version information\n"
    "\t    --worldbg=R,G,B     background radiance W [0,inf)\n"
    "\t    --windowbg=R,G,B    interactive viewer background color [0,1]\n"
    "\t    --sigma_t=NUM       scattering coefficient in volume scattering (0.5)\n"
    "\t    --albedo=NUM        scattering albedo in volume scattering (0.5)\n"
    "\t    --stepsize=NUM      step size for volume scattering (experimental) (0.0)\n"
    "\t    --3ddata=FILE       scalar data filename\n"
    "\t    --3ddata2=FILE      second data filename\n"
    "\t    --3ddim=NUMxNUMxNUM size of single dimension of data (100x100x100)\n"
    "\t    --3dheader=NUM      bytes to skip in data file (512)\n"
    "\t    --3dmult=NUM        multiplier to scale data by (1.0)\n"
    "\t    --3dmult2=NUM       multiplier to scale 2nd data by (1.0)\n"
    "\t    --3dmax=X,Y,Z       spatial size of dataset (100, 100, 100)\n"
    "\t    --3dcolor=R,G,B     color of dataset (1, 1, 1)\n"
    "\n"
    ;
}
