////////////////////////////////////////////////////////////////////////////
//
//WW''''WW''''''WW''''''''''''''WWW''''''''WW''''''''WWW'''''''
//WWW  WWW                       WW        WW         WW       
//W WWW WW  WWW WW WWW      WWW  WW   WW   WWW   WWW  WW   WWWW
//W  W  WW W WW WW WW W    W WW  WW  WW W  WW W W WW  WW  WWW  
//W     WW W WW WW WW W    W WW  WW  WW W  WW W W WW  WW    WWW
//W     WW  WWW WW WW W     WWW WWWW  WW   WWW   WWW WWWW WWWW 
//                           WW                                
//'''''''''''''''''''''''''WWW'''''''''''''''''''''''''''''''''
//                                                                           
// File: Main.h
// Description: Global includes, variables, and function definitions
// Created: 2003/07/17 09:33:17
// Author: Kevin Beason <beason@cs.fsu.edu>
//
// Copyright (c) 2003-2004 Kevin Beason
// 
// Main header for the Pane ray tracer.
// For more information on Pane, visit:
// http://www.csit.fsu.edu/~beason/pane/
// or see the accompanying documentation
// in the distribution.
//
// Usage: pane [OPTION]... [IV FILE]
//
// Synopsis: pane is a ray tracer.
//           Input is a scene in the Open Inventor format (scene.iv)
//           Output is an image (scene.png).
//
// $Id: Main.h 1188 2007-12-08 18:23:02Z beason $
//
////////////////////////////////////////////////////////////////////////////

#ifndef MAINGLOBALS
#define MAINGLOBALS

#include <cstdio>
#include <cstdlib>
#include <unistd.h>
#include <sys/time.h>
#include <sys/times.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <cmath>
//#include <mathimf.h>
#include <iostream>
#include <iomanip>
#include <fstream>
#include <list>
#include <string>
#include <errno.h>
#include <vector>
#include <getopt.h>
#include <string.h>
#include <pthread.h>
#include <semaphore.h>
#include <regex.h>
#include <sstream>
#include <map>

// environ on OS X
#ifdef __darwin__
# include <crt_externs.h>
# define environ (*_NSGetEnviron())
#endif

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/engines/SoElapsedTime.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/SoPrimitiveVertex.h>
#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/SbViewportRegion.h>
#include <Inventor/SoOutput.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/nodes/SoTriangleStripSet.h>
#include <Inventor/nodes/SoResetTransform.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/sensors/SoNodeSensor.h>
#include <Inventor/nodes/SoPickStyle.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/actions/SoGetMatrixAction.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoInfo.h>
#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoFile.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoText3.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoLOD.h>
#include <Inventor/SoInteraction.h>
#include <Inventor/nodes/SoTextureCoordinate2.h>

// memory leak detector
#ifdef DEBUG_MEM
#include "memory_debug.h"
#endif 

// random number generator
#include "RNG.h"

#ifdef SUNOS
#include <alloca.h>
#include <ieeefp.h>

inline int isinf(double x) { return !finite(x) && x==x; }

#ifdef PNG_SETJMP_SUPPORTED
#  define png_jmpbuf(png_ptr) ((png_ptr)->jmpbuf)
#else
#  define png_jmpbuf(png_ptr) \
   (LIBPNG_WAS_COMPILED_WITH__PNG_SETJMP_NOT_SUPPORTED)
#endif

char* strsep(char** str, const char* delims);

#endif /* SUNOS settings */

using namespace std;

//namespace paneMain {

extern int DEBUG_STATE;                         // defined in Main.cxx
extern int STOP_RENDERING;                      // defined in Main.cxx
extern SoSeparator *scnRoot;                    // defined in Main.cxx
extern RNG mydrand48;                           // defined in Main.cxx
extern const int mutexNum;                      // defined in Main.cxx
extern pthread_mutex_t mutexes[];               // defined in Main.cxx

enum { X, Y, Z };                               // defs for XYZ
enum { R, G, B, E };                            // defs for RGBExponent
enum { LOCK_STDERR, LOCK_RAND, LOCK_ICACHE, LOCK_ICREJECTS, LOCK_ENV }; // thread locks
enum { HL_OFF, HL_CENTER, HL_OFFCENTER };


// find min/max of three/two vals
// Erkk! Make these inline! Or else they could be executing arguments twice.
#define MIN3(a,b,c) ((((a)<(b))&&((a)<(c))) ? (a) : (((b)<(c)) ? (b) : (c)))
#define MAX3(a,b,c) ((((a)>(b))&&((a)>(c))) ? (a) : (((b)>(c)) ? (b) : (c)))
#define MAX2(a,b) ( ((a)>(b)) ? (a) : (b) )
#define MIN2(a,b) ( ((a)<(b)) ? (a) : (b) )

// vector mul and finite
SbVec3f mul(SbVec3f &x, SbVec3f &y, float c=1.0);
int finite(SbVec3f &c);
  
// find absolute value
#define ABSF(x) ( ((x)<0.0) ? -(x) : (x) )

// time a set of instructions
#define TIME(x) { struct timeval start, end;  \
                gettimeofday(&start, NULL); \
                x;                          \
 	        gettimeofday(&end, NULL);   \
	        cerr << "Time: "  << delay(start, end) << "ms" << endl; }


// brightness with weights tuned to human visual system
inline float brightness(SbColor const &c){
  return(0.27*c[R] + 0.67*c[G] + 0.06*c[B]);
}
inline float brightness(const unsigned char *i){
  return((0.27*i[0] + 0.67*i[1] + 0.06*i[2])/255.0);
}

// structure for threads maintaining state.
// was in Pane.h, moved to here
// so it can be used in Ray.h as well.
typedef struct thread_context_st {
  RNG mydrand48;  // random number generator
  int id;         // thread id
  int rays;       // rays counter
  int isects;     // intersections counter
  clock_t ticks;  // ticks for thread
  void *other;    // for passing miscellaneous data
} thread_context;

// clamp definition
// restricts a number to a range [min, max]
template <class T> inline
T clamp(T x, T min, T max){
  if (x<min)
    return(min);
  else if (x>max)
    return(max);
  else
    return(x);
}

// Compute the delay between t1 and t2 in milliseconds
inline long delay (struct timeval t1, struct timeval t2){
  long d;
    
  d = (t2.tv_sec - t1.tv_sec) * 1000;
  d+= ((t2.tv_usec - t1.tv_usec + 500) / 1000);
  return(d);
}

inline void resetios(ostream &s){
  const int defaultPrc = 6;
  s << resetiosflags(ios::floatfield) << setprecision(defaultPrc);
}

inline int nearestPowerOf2(unsigned int x){
  unsigned int i=0, y=x;                                                                           
  while (y=y>>1) i++;                                                                 
  int r = 1<<i;                                                                       
  if (r!=x)                                                                           
    r = r<<1;
  return r;
}

template <typename T>
inline void vcopy2(T *dst, T *src){
  dst[0] = src[0];
  dst[1] = src[1];
}

template <typename T>
inline void vcopy3(T *dst, T *src){
  dst[0] = src[0];
  dst[1] = src[1];
  dst[2] = src[2];
}


                   
//}

#endif /* MAINGLOBALS */


