// Header for Misc.cxx

#ifndef MISCFUNCS
#define MISCFUNCS

#include <Inventor/Xt/SoXt.h>
#include <Inventor/Xt/viewers/SoXtExaminerViewer.h>
#include <Inventor/engines/SoElapsedTime.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoCone.h>
#include <Inventor/nodes/SoDirectionalLight.h>
#include <Inventor/nodes/SoMaterial.h>
#include <Inventor/nodes/SoPerspectiveCamera.h>
#include <Inventor/nodes/SoOrthographicCamera.h>
#include <Inventor/nodes/SoSeparator.h>
#include <Inventor/SoInput.h>
#include <Inventor/nodes/SoCamera.h>
#include <Inventor/events/SoKeyboardEvent.h>
#include <Inventor/nodes/SoEventCallback.h>
#include <Inventor/SoPrimitiveVertex.h>
#include <Inventor/actions/SoCallbackAction.h>
#include <Inventor/nodes/SoTransform.h>
#include <Inventor/nodes/SoSphere.h>
#include <Inventor/nodes/SoCube.h>
#include <Inventor/SbLinear.h>
#include <Inventor/nodes/SoPointLight.h>
#include <Inventor/nodes/SoIndexedLineSet.h>
#include <Inventor/nodes/SoIndexedFaceSet.h>
#include <Inventor/nodes/SoCoordinate3.h>
#include <Inventor/nodes/SoCylinder.h>
#include <Inventor/SbViewportRegion.h>
#include <Inventor/SoOutput.h>
#include <Inventor/actions/SoWriteAction.h>
#include <Inventor/nodes/SoTriangleStripSet.h>
#include <Inventor/nodes/SoResetTransform.h>
#include <Inventor/sensors/SoTimerSensor.h>
#include <Inventor/sensors/SoNodeSensor.h>
#include <Inventor/nodes/SoPickStyle.h>
#include <Inventor/nodes/SoDrawStyle.h>
#include <Inventor/nodes/SoRotation.h>
#include <Inventor/actions/SoSearchAction.h>
#include <Inventor/nodes/SoGroup.h>
#include <Inventor/nodes/SoSwitch.h>
#include <Inventor/nodes/SoMaterialBinding.h>
#include <Inventor/nodes/SoLightModel.h>
#include <Inventor/actions/SoGetMatrixAction.h>
#include <Inventor/nodes/SoIndexedTriangleStripSet.h>
#include <Inventor/nodes/SoInfo.h>
#include <Inventor/nodes/SoComplexity.h>
#include <Inventor/nodes/SoFile.h>
#include <Inventor/nodes/SoText2.h>
#include <Inventor/nodes/SoText3.h>
#include <Inventor/nodes/SoFont.h>
#include <Inventor/nodes/SoNormal.h>

#include <iostream>

using namespace std;

int computeNormals(SoNode *scene);


//'WWW''WW'''W'WWWW''''''''''WWW''''''''''''''''''
//WW  W WWW  W WW  W          WW                  
//WW  W W WW W WWWW      WWW  WW   WWW  WWWW  WWWW
//WW  W W  WWW WW  W    WW W  WW  W WW WWW   WWW  
//WW  W W   WW WW  W    WW    WW  W WW   WWW   WWW
// WWW  W    W WWWW      WWW WWWW  WWW WWWW  WWWW 
//                                                
//''''''''''''''''''''''''''''''''''''''''''''''''

// ortho-normal basis class
class ONB {

private:
  
  SbVec3f u, v, w;

public:
  
  void init(){
    u.setValue(1,0,0);
    v.setValue(0,1,0);
    w.setValue(0,0,1);
  }
  
  void constructFromUV(SbVec3f &a, SbVec3f &b){
    u = a;
    u.normalize();

    v = a.cross(b);
    v.normalize();

    w = v.cross(u);
  }

  SbVec3f toONBSpace(const SbVec3f &a){
    SbVec3f x;
    x[0] = u.dot(a);
    x[1] = v.dot(a);
    x[2] = w.dot(a);
    return(x);
  }

  SbVec3f toWorldSpace(const SbVec3f &a){
    return SbVec3f(a[0]*u+a[1]*v+a[2]*w);
  }

};

#endif /* MISCFUNCS */
