//WWWW''''''''''''''''''''WWW''''''''''''''''''
//WW  W                    WW                  
//WW  W  WWW W WW     WWW  WW   WWW  WWWW  WWWW
//WWWW  W WW W WW    WW W  WW  W WW WWW   WWW  
//WW  W W WW W WW    WW    WW  W WW   WWW   WWW
//WW  W  WWW  WWW     WWW WWWW  WWW WWWW  WWWW 
//             WW                              
//'''''''''''WWW'''''''''''''''''''''''''''''''

#include "Ray.h"
#include "debug.h"

namespace {
  pthread_mutex_t lock_show = PTHREAD_MUTEX_INITIALIZER;
  pthread_mutex_t lock_show2 = PTHREAD_MUTEX_INITIALIZER;
}

int Ray::NO_TRI = INT_MIN;

// cylinder
void Ray::show(double height, SbColor color, int clear){
  static double hue = 0.0;
  double red, green, blue;

  // height stuff
  double cb_rat = 0.05;
  double cone_height = cb_rat*height;
  double bar_height = height-cone_height;
  double bar_radius = bar_height/100.0;
  double cone_radius = bar_height/30.0;

  // figure mode
  /*
  cb_rat = 0.05;
  cone_height = 4.0;
  bar_height = height-cone_height;
  bar_radius = 1.0;
  cone_radius = bar_radius*3.0;
  */
  
  // add self to scene graph
  pthread_mutex_lock(&lock_show);
  static SoSeparator *showRaySep = NULL;
  if (showRaySep == NULL){
    // add cube sep to scene
    showRaySep = new SoSeparator();
    SoComplexity *comp = new SoComplexity();
    comp->value.setValue(0.3);
    showRaySep->addChild(comp);
    scnRoot->addChild(showRaySep);
  }
  else if (clear){
    showRaySep->removeAllChildren();
    SoComplexity *comp = new SoComplexity();
    comp->value.setValue(0.3);
    showRaySep->addChild(comp);

    pthread_mutex_unlock(&lock_show);
    return;
  }
  pthread_mutex_unlock(&lock_show);
  
  // Ray debugging
  SoSeparator *sep = new SoSeparator();
  SoTransform *trn;
  SoMaterial *mat = new SoMaterial();
  mat->ambientColor.setValue(0.0, 0.0, 0.0);
  mat->diffuseColor.setValue(0.0, 0.0, 0.0);
  mat->specularColor.setValue(1.0, 1.0, 1.0);
  mat->shininess.setValue(0.3);
  mat->transparency.setValue(0.5);

  // figures
  if (0){
    mat->ambientColor.setValue(0.2, 0.2, 0.2);
    mat->diffuseColor.setValue(0.8, 0.8, 0.8);
    mat->specularColor.setValue(0.0, 0.0, 0.0);
    mat->shininess.setValue(0.1);
    mat->transparency.setValue(0.0);
  }

  if (color[0]<0.0){
    //printCol(color);

    pthread_mutex_lock(&lock_show);
    hue += (1.0-hue)*0.05;
    mat->emissiveColor.setHSVValue(hue, 1.0, 0.8);
    pthread_mutex_unlock(&lock_show);
  }
  else {
    for (int i=0; i<3; i++)
      color[i] = 1.0-exp(-2.0*color[i]/10.0);
//      color[i] = clamp(color[i], 0.0f, 1.0f);
    mat->diffuseColor.setValue(color);
  }
  sep->addChild(mat);
    
  // Translate base
  trn = new SoTransform();
  trn->translation.setValue(pos);
  sep->addChild(trn);

  
  // Scale
  trn = new SoTransform();

  // Rotate
  SbRotation sbr;
  sbr.setValue(SbVec3f(0.0, 1.0, 0.0), dir);
  trn->rotation.setValue(sbr);
  sep->addChild(trn);

  // Pole
  SoCylinder *cyl = new SoCylinder();
  cyl->radius.setValue(bar_radius);
  cyl->height.setValue(bar_height);
  trn = new SoTransform();
  trn->translation.setValue(SbVec3f(0.0, bar_height/2.0, 0.0));
  sep->addChild(trn);
  sep->addChild(cyl);
  
  // Cone
  SoCone *con = new SoCone();
  con->bottomRadius.setValue(cone_radius);
  con->height.setValue(cone_height);
  trn = new SoTransform();
  trn->translation.setValue(SbVec3f(0.0, height/2.0, 0.0));
  sep->addChild(trn);
  sep->addChild(con);
  
  pthread_mutex_lock(&lock_show);
  showRaySep->addChild(sep);
  pthread_mutex_unlock(&lock_show);
}

class LineSet {
private:
  SoCoordinate3 *coords;
  SoIndexedLineSet *index;
  SoSeparator *sep;
  SoMaterial *mat;
  SoMaterialBinding *bind;
  int numCoords, numIndexes;
  
public:
  LineSet(){
    coords = new SoCoordinate3();
    index = new SoIndexedLineSet();
    bind = new SoMaterialBinding();
    bind->value = SoMaterialBinding::PER_VERTEX_INDEXED;
    mat = new SoMaterial();
    sep = new SoSeparator();
    mat->diffuseColor.set1Value(0, SbColor(1.0, 0.0, 0.0)); // red
    mat->diffuseColor.set1Value(1, SbColor(1.0, 1.0, 1.0)); // white
    coords->point.set1Value(0, SbVec3f(0.0, 0.0, 0.0));
    coords->point.set1Value(1, SbVec3f(0.0, 0.0, 0.0));
    index->coordIndex.set1Value(0, 0);
    index->coordIndex.set1Value(1, 1);
    index->coordIndex.set1Value(2, -1);
    numCoords = 2;
    numIndexes = 3;
    sep->addChild(bind);
    sep->addChild(mat);
    sep->addChild(coords);
    sep->addChild(index);
  }

  SoSeparator *getSep(){
    return(sep);
  }
      
  void draw(const SbVec3f &a, const SbVec3f &b){
    // set coords
    coords->point.set1Value(numCoords++, a);
    coords->point.set1Value(numCoords++, b);

    // set index
    index->coordIndex.set1Value(numIndexes++, numCoords-2);
    index->coordIndex.set1Value(numIndexes++, numCoords-1);
    index->coordIndex.set1Value(numIndexes++, -1);

    // set color
    mat->diffuseColor.set1Value(numCoords-2, SbColor(0.0, 0.0, 1.0));
    mat->diffuseColor.set1Value(numCoords-1, SbColor(1.0, 1.0, 0.0));
  }

  void draw(const SbVec3f &a, const SbVec3f &b, const SbColor &c){
    // set coords
    coords->point.set1Value(numCoords++, a);
    coords->point.set1Value(numCoords++, b);

    // set index
    index->coordIndex.set1Value(numIndexes++, numCoords-2);
    index->coordIndex.set1Value(numIndexes++, numCoords-1);
    index->coordIndex.set1Value(numIndexes++, -1);

    // set color
    mat->diffuseColor.set1Value(numCoords-2, SbColor(0.0, 1.0, 1.0));
    //mat->diffuseColor.set1Value(numCoords-2, c);
    mat->diffuseColor.set1Value(numCoords-1, c);
  }

  void draw1(const SbVec3f &a, const SbVec3f &b, const SbColor &c){

    // set coords
    coords->point.set1Value(0, a);
    coords->point.set1Value(1, b);

    // set color
    mat->diffuseColor.set1Value(1, c);
  }

};

LineSet * Ray::ls;

// Line set
void Ray::show2(double height, SbColor color, int clear){
  SbVec3f tip = pos+height*dir;

  static double hue = 0.0;
  double red, green, blue;
  
  pthread_mutex_lock(&lock_show2);

  // allocate LineSet
  if (ls == NULL)
    ls = new LineSet();
  
  // add self to scene graph
  static SoSeparator *showRaySep = NULL;
  if (showRaySep == NULL){
    // add cube sep to scene
    showRaySep = new SoSeparator();
    showRaySep->addChild(ls->getSep());
    scnRoot->addChild(showRaySep);
  }
  else if (clear){
    showRaySep->removeAllChildren();
    return;
  }

  if (color[0]<0.0){
    //printCol(color);
    hue += (1.0-hue)*0.05;
    color.setHSVValue(hue, 1.0, 0.8);
  }
  
  ls->draw(pos, tip, color);
  pthread_mutex_unlock(&lock_show2);
}

ostream& operator<< (ostream& os, const Ray& r){
  os << setiosflags(ios::showpoint | ios::fixed) << setprecision(8);
  os << "Ray:(" << r.pos << ", " << r.dir << ") ";
  resetios(os);
  return(os);
}
