//WWWW''''''''''''''''''''WWW''''''''''''''''''
//WW  W                    WW                  
//WW  W  WWW W WW     WWW  WW   WWW  WWWW  WWWW
//WWWW  W WW W WW    WW W  WW  W WW WWW   WWW  
//WW  W W WW W WW    WW    WW  W WW   WWW   WWW
//WW  W  WWW  WWW     WWW WWWW  WWW WWWW  WWWW 
//             WW                              
//'''''''''''WWW'''''''''''''''''''''''''''''''

#ifndef RAYCLASS
#define RAYCLASS

#include "Main.h"

// for piggyback variables
class Triangle;  // forward declaration
class LineSet;

class Ray {
private:

  static LineSet *ls;
  
public:

  static int NO_TRI;
  
  SbVec3f pos;
  SbVec3f dir;  // must be normal (unit length)

  int depth;               // depth from eye of hit point (first is 1)
  thread_context *ctx;     // context for access to thread global vars/RNG's
  SbColor *col;            // color pointer, for debugging with gdb

  // new
  int atten;               // True if in inside dielectric and Beer's law should be applied
  int inside;
  float time;              // time of ray
  int i;                   // instance i
  
  //Triangle *orig;          // origin of ray (if it exists)
  float importance;        // importance of result of ray
                           // (1.0 == very important, primary)
  double tMax;             // for limiting tracing of ray

  // piggy back variables
  double t;                // length along ray to hit point
  double tMedia;           // length until first participating media scatter
  Triangle *tri;           // pointer to triangle that this ray hits
  int triId;               // id of triangle that ray hits
  int ignoreId;            // id of triangle ray should not hit, if >=0
  SbVec3f x;               // position in space of the hit point
  SbVec3f bc;              // barycentric coordinates of hit point
  SbColor diffCol;         // diffuse color of hit point surface

  Ray() : pos(0,0,0), dir(0,0,-1), importance(1), atten(0), inside(0), ignoreId(NO_TRI) {
    // pos.setValue(0.0, 0.0, 0.0);
    // dir.setValue(0.0, 0.0, -1.0);
    // importance = 1.0;
  }

  Ray(SbVec3f _pos, SbVec3f _dir, thread_context *_ctx):
    pos(_pos), dir(_dir), ctx(_ctx), importance (1.0), atten(0), inside(0), ignoreId(NO_TRI) { }
  
  Ray(SbVec3f _pos, SbVec3f _dir, thread_context *_ctx, int _depth):
    pos(_pos), dir(_dir), ctx(_ctx), depth(_depth), importance(1.0), atten(0), inside(0), ignoreId(NO_TRI) { }
  
  Ray(SbVec3f _pos, SbVec3f _dir, thread_context *_ctx, int _depth, int _ignoreId):
    pos(_pos), dir(_dir), ctx(_ctx), depth(_depth), importance(1.0), atten(0), inside(0), ignoreId(_ignoreId) { }

  Ray(SbVec3f _pos, SbVec3f _dir, thread_context *_ctx, int _depth, int _atten, int _inside):
    pos(_pos), dir(_dir), ctx(_ctx), depth(_depth), importance(1.0), atten(_atten), inside(_inside), ignoreId(NO_TRI) { }

  Ray(SbVec3f _pos, SbVec3f _dir, thread_context *_ctx, int _depth, int _atten, int _inside, int _ignoreId):
    pos(_pos), dir(_dir), ctx(_ctx), depth(_depth), importance(1.0), atten(_atten), inside(_inside), ignoreId(_ignoreId) { }
  
  // returns point that is parameter t along the ray
  SbVec3f point(float t){
    return(pos+t*dir);
  }

  void ignore(int i){ ignoreId = i; }

  // draws cylinder representation
  void show(double height = 100.0, SbColor color = SbColor(-1.0, -1.0, -1.0),
	    int clear = 0);

  
  // draws line representation
  void show2(double height = 100.0, SbColor color = SbColor(-1.0, -1.0, -1.0),
	     int clear = 0);

  friend ostream& operator<< (ostream& os, const Ray& r);
};

#endif /* RAYCLASS */
