//WWWW'''''''WW''''''''''''''''''WW''''''''''''''WW''W'''WW'''''
//WW  W      WW                                  WW  W   WW     
//WW  W  WW  WWW  W WW  WWW  WWW WW WWW   WWW    WW  W  WWW W WW
//WW  W WWWW WW W W WW W WW W WW WW WW W W WW    WWWWW W WW WW  
//WW  W WW   WW W W WW W WW W WW WW WW W W WW    WW  W W WW WW  
//WWWW   WWW WWW   WWW  WWW  WWW WW WW W  WWW    WW  W  WWW WW  
//                       WW   WW           WW                   
//'''''''''''''''''''''WWW''WWW''''''''''WWW''''''''''''''''''''

#ifndef DEBUGFUNCS
#define DEBUGFUNCS

#include <stdarg.h>
#include <signal.h>
#include <fenv.h>
#include <string>
#include "Main.h"

namespace {
  pthread_mutex_t lock_printVec = PTHREAD_MUTEX_INITIALIZER;
}

inline static void printCol(SbColor x){
  pthread_mutex_lock(&lock_printVec);
  cerr << "(";
  cerr << setiosflags(ios::showpoint | ios::fixed) << setprecision(8);
  cerr << x[X] << ", ";
  cerr << x[Y] << ", ";
  cerr << x[Z] << ") " << flush;
  resetios(cerr);
  pthread_mutex_unlock(&lock_printVec);
  return;
}

inline static void printVec(SbVec3f x){
  printCol(SbColor(x));
}

ostream& operator<< (ostream& os, const SbVec3f& x);

//#define DBG(s, ...) fprintf(stderr, "BEASON: %s:%d: %s\n", __FILE__, __LINE__, s, __VA_ARGS__);
#define DBG(s, ...) fprintf(stderr, "BEASON: %s:%d: " s "\n", __FILE__, __LINE__, ## __VA_ARGS__ );

void complain(int line);
void showPoint(SbVec3f p, double radius, SbColor c = SbColor(-1,-1,-1), int clear=0);
long delay (struct timeval t1, struct timeval t2);
void showCube(SbVec3f min, SbVec3f max, SbVec3f c, int clear=0);
void showPhoton(SbVec3f p, SbColor c = SbColor(1.0, 1.0, 1.0), int clear=0);
void print(char const *fmt, ...);
void drawHist(float *flux_map, int n_phi, int n_theta, float scale,
	      SbVec3f x, SbVec3f n);
void backTrace(bool both=false);

// These functions must be fast because they are used
// in intersect_ray routines.
inline void disableExceptions(){
#ifndef __darwin__
    fedisableexcept(FE_ALL_EXCEPT);
#endif
}

inline void enableExceptions(){
#ifndef __darwin__
  feclearexcept(FE_ALL_EXCEPT);
  extern int ENABLED_FPES;
  feenableexcept(ENABLED_FPES);
#endif
}

void setupSignals();
void signalHandler(int sig);
void sigactionHandler(int sig, siginfo_t *info, void *context);
string& getProgramName();

#endif /* DEBUGFUNCS */
