// Title  : pfmtohdr
// Purpose: Converts portable float map (.pfm) file to Radiance format (.hdr)
// Usage  : pfmtohdr <infile.pfm> <outfile.hdr>
// Compile: g++ pfmtohdr.cxx -o obj/pfmtohdr obj/rgbe.o
// Author : Kevin Beason 6/26/07

#include <stdio.h>
#include <errno.h>
#include <iostream>
#ifdef __darwin__
#include <machine/endian.h>
#else
#include <endian.h>
#endif
#include "FileUtils.h"

using namespace std;

int main(int argc, char *argv[]){

  const bool verbose = false;
  
  if (argc<3) {
    printf("Usage: pfmtohdr <infile.pfm> <outfile.png>\n",
           argv[0]);
    return 1;
  }

  int dim[2];
  float *depthRGB = FileUtils::readPFM(dim, argv[1], verbose);
  if (!depthRGB){
    printf("Error reading PFM\n");
    return 1;
  }

  // scale depth to be [0,1]
  int s = dim[0]*dim[1]*3;
  float min[3] = { 1e99, 1e99, 1e99};
  float max[3] = {-1e99,-1e99,-1e99};
  for (int i=0; i<s; i++){
    if (depthRGB[i]<min[0])
      min[0]=depthRGB[i];
    if (depthRGB[i]>max[0])
      max[0]=depthRGB[i];
  }
  for (int i=0; i<s; i++)
    depthRGB[i] = (depthRGB[i]-min[0]) / (max[0]-min[0]);
  
  if (verbose) printf("Writing %s...", argv[2]);
  FileUtils::writePNG(dim, argv[2], (const SbColor*)depthRGB, 2.2, "pfmtopng");
  if (verbose) printf("done.\n");
  
  return 0;
}
